/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.components;

import java.io.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.resources.*;
import talon.util.net.*;

import javax.xml.transform.*;

import org.openprivacy.panther.event.*;

/**
 * Remote proxy interface for URL/resource caching/fetching.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
 */
public interface Proxy extends Component {

    /**
     * Get a resource known to this Proxy.  This uses a Talon ResourceMapper to
     * obtain resources.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public Resource getResource( URI uri ) throws IOException;

    /**
     * @see #getResource
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public Resource getResource( String uri ) throws IOException;

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void setResourceMapper( ResourceMapper mapper );

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public ResourceMapper getResourceMapper();

    /**
     * Get the given URL from the cache.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry get( String location ) throws IOException;

    /**
     * @see #get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry get( URI location ) throws IOException;

    /**
     * Get a URL from the remote site and wait until it is in the cache.  If it
     * is already in the cache. return the local version.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public ProxyEntry getAndWait( URI location ) throws Exception;

    /**
     * Get a URL from the remote site and wait until it is in the cache.  If it
     * is already in the cache. return the local version.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public ProxyEntry getAndWait( String location ) throws Exception;
    
    /**
     * Request that the Proxy update a URL.  Note that due to optimization a
     * Proxy might thread this off.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void update( String location ) throws IOException;

    /**
     * @see #update
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void update( URI location ) throws IOException;

    /**
     * Sometimes it is necessary to update content from within one thread but we
     * don't care if it was actually done or not.  We just want to signal that
     * an update should proceed.  The async update method tells the Proxy to
     * fetch a URL if it is not available locally.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void asyncUpdate( String location );

    /**
     * @see #asyncUpdate
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void asyncUpdate( URI location );

    /**
     * Return true if this proxy supports the given URI.  
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public boolean supports( URI location );

    /**
     * @see #supports
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public boolean supports( String location );
    
    /**
     * Return true if the given URL is within the cache. 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public boolean contains( URI location );

    /**
     * @see #contains
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public boolean contains( String location );

    /**
     * Get a URIResolver for use with TrAX.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public URIResolver getURIResolver();

    /**
     * Add a ProxyEventListener for observing proxy activity.
     *
     * @see ProxyEventListener
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void addProxyEventListener( ProxyEventListener listener );

    /**
     * Remove an EventListener.
     * 
     * @see ProxyEventListener
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Proxy.java,v 1.15 2002/03/08 21:42:54 burton Exp $
     */
    public void removeProxyEventListener( ProxyEventListener listener );


    /**
     * Get a URL from the remote site and wait until it is in the cache.  If it
     * is already in the cache. return the local version.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public ProxyEntry updateAndWait( URI location ) throws Exception;

    /**
     * Get a URL from the remote site and wait until it is in the cache.  If it
     * is already in the cache. return the local version.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public ProxyEntry updateAndWait( String location ) throws Exception;
    
}
