/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.components;

import java.io.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.resources.*;
import talon.util.net.*;

/**
 * Represents an fetched from the proxy.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: ProxyEntry.java,v 1.9 2002/02/26 01:38:34 burton Exp $
 */
public interface ProxyEntry {

    /**
     * Get an InputStream for reading from this ProxyEntry.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public InputStream getInputStream() throws IOException;

    /**
     * <p>
     * Get the URI that was used to request this ProxyEntry.  
     *
     * <p>
     * Examples:
     *
     * <ul>
     *    <li>http://www.openprivacy.org</li>
     *    <li>resource:/xml/users/default/subscriptions.xml</li>
     * </ul>
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public URI getIdentifier();

    /**
     * Return the last time this URL was modified within the cache.  The result
     * is the number of milliseconds since January 1, 1970 GMT.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public long getLastModified();

    /**
     * Get the last message generated by the proxy for this URL.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public String getLastMessage();

    /**
     * @see #getLastMessage
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public void setLastMessage( String lastMessage );

    /**
     * Get the file:/ URL which points to this URL on disk.  This is the data
     * file which contains the cached content of this URL.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @deprecated This will be changed in the future to return a URI.
     */
    public String getFileLocation();

    /**
     * Get the File which this ProxyEntri is backing up.  This will throw an
     * IOException if the file is invalid or this is based on a Resource
     * (resources aren't backed on disk).
     *
     * @see #getFileLocation
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public File getFile() throws IOException;
    
    /**
     * Return true if the URL that this ProxyEntry backs has changed.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public boolean hasChanged();

    /**
     * Get the minimum amount of time that this entry should remain within the
     * cache before being updated.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public long getMinCachedInterval();

    /**
     * @see #getMinCachedInterval
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public void setMinCachedInterval( long interval );

    /**
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public void setResource( boolean resource );

    /**
     * 
     * Get the value of <code>resource</code>.
     * 
     * This is true if this ProxyEntry is based on a resource.
     * 
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public boolean isResource();

    /**
     * Save this ProxyEntry to disk.  This is used if you want to make any
     * modifications for this entry so that future applications can access them.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     */
    public void save() throws IOException;

    /**
     * Return the mimeType for this ProxyEntry.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public MimeType getMimeType();

    /**
     * Set the mimeType for this ProxyEntry.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public void setMimeType( MimeType mt );

}

