/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.event;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;

/**
 * Handles registering ProxyEventListeners and firing them off.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: ProxyEventManager.java,v 1.1 2001/08/09 23:55:30 burton Exp $
 */
public class ProxyEventManager {
    
    /**
     * Instance member for <code>ProxyEventManager</code>
     */
    private static ProxyEventManager instance = null;

    private Vector listeners = new Vector();

    /**
     * 
     * Get an instance of the <code>ProxyEventManager</code>.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEventManager.java,v 1.1 2001/08/09 23:55:30 burton Exp $
     */
    public static ProxyEventManager getInstance() {
        
        if ( instance == null ) {
            
            instance = new ProxyEventManager();
            
        }
        
        return instance;
        
    }
    
    /**
     * Add an EventListener.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEventManager.java,v 1.1 2001/08/09 23:55:30 burton Exp $
     */
    public void addProxyEventListener( ProxyEventListener listener ) {

        this.listeners.addElement( listener );
        
    }

    /**
     * Remove an EventListener.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEventManager.java,v 1.1 2001/08/09 23:55:30 burton Exp $
     */
    public void removeProxyEventListener( ProxyEventListener listener ) {

        this.listeners.removeElement( listener );
        
    }
    

    /**
     * Fire all listners and notify them that they have been updated.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEventManager.java,v 1.1 2001/08/09 23:55:30 burton Exp $
     */
    public void fireUpdated( ProxyEntry entry ) {

        Enumeration elements = listeners.elements();

        while ( elements.hasMoreElements() ) {

            ProxyEventListener current = (ProxyEventListener)elements.nextElement();

            current.updated( new ProxyEvent( entry ) );
            
        } 
        
    }
    
}
