/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;


import java.io.*;
import java.util.*;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.util.*;


//TODO: 

// - Move this class into an implementation package.
//
// - Maybe this class should become a CacheNodeManager ???

/**
 * Responsible for setting up the necessary directories and XML content for this
 * cache entry.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
 */
public class CacheConstructor {

    /**
     * Handles quickly mapping URLs to their location in the cache.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
     */
    private static Hashtable cacheNodes = new Hashtable();
    
    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
     */
    public static CacheNode construct( URI location ) throws IOException {

        //see if we have found/constructed this cache entry/node before while
        //Panther was running.

        CacheNode node = getCacheNode( location.toString() );

        if ( node != null ) {

            return node;

        } 
        
        String root = System.getProperty( "panther.root" );

        if ( root == null ) {

            //default to the users tmpdir
            root = System.getProperty( "java.io.tmpdir" ) + "/panther/cache/";
            
        } 

        root = new File( root ).getAbsolutePath() + "/"; 
        
        //add the scheme root.
        root = root + location.getScheme() + "/";

        //just make sure at least these are ready...

        //construct the host portion
        root = constructHostPortion( root, location );

        root = constructPathPortion( root, location );

        root = constructXML( root );

        node = new CacheNode( root, true );
        
        //set the cache node as this was not set before
        setCacheNode( location.toString(), node );
        
        return node;
    }

    public static CacheNode construct( String location ) throws IOException {

        return construct( new URI( location ) );

    }

    /**
     * Construct the host portion of the directory and return the last created
     * directory path
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
     */
    private static final String constructHostPortion( String root, URI location ) throws IOException {

        String hostname = location.getURL().getHost();

        //now go through all paths and create their directories.
        StringTokenizer tok = new StringTokenizer( hostname, "." );

        while ( tok.hasMoreTokens() ) {

            String token = tok.nextToken();

            root = root + token + "/";

        } 

        return root;

    }    

    /**
     * Construct the path portion of the cache.  Return the last created
     * directory path.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
     */
    private static final String constructPathPortion( String root, URI location ) throws IOException {

        String scheme = location.getScheme();

        String host = location.getURL().getHost();

        int start = scheme.length() + "::/".length() + host.length();
        
        int end = location.toString().lastIndexOf( "/" );
        
        if ( end < 0 || start > end ) {
            //no path necessary.

            new File( root ).mkdirs();

            return root;
            
        } 
        
        String path = location.toString().substring( start, end );
        
        //now go through all paths and create their directories.
        StringTokenizer tok = new StringTokenizer( path, "/" );

        String current = root;

        while ( tok.hasMoreTokens() ) {

            String token = tok.nextToken();

            current = current + token + "/";

        } 

        new File( current ).mkdirs();

        return current;
    }

    /**
     * Make sure that data.xml exists in the given cache directory.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheConstructor.java,v 1.6 2002/03/08 21:42:54 burton Exp $
     */
    public static final String constructXML( String root ) throws IOException {

        String filename = root + "/" + PantherResources.DATA_XML;
        
        //if data.xml doesn't exist create it.
        if ( ! new File( filename ).exists() ) {

            PrintWriter pw = new PrintWriter( new FileWriter( filename ) );

            //NOTE: I hate embedding code but this works well.
            pw.println( "<cache xmlns=\"" + PantherResources.CACHE_NAMESPACE + "\">" );

            pw.println( "</cache>" );

            pw.close();
            
        } 

        return filename;
        
    }

    public static final CacheNode getCacheNode( String location ) {

        return (CacheNode)cacheNodes.get( location );
        
    }

    public static final void setCacheNode( String location, CacheNode node ) {

        cacheNodes.put( location, node );
        
    }
    
}

