/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.util.*;

/**
 * Represents a cache node (data.xml) for storing cache data.  This also
 * includes metainfo for use within Panther.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
 */
public class CacheNode {

    private boolean isNew = false;

    private String path = "";

    private String directory = "";
    
    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
     */
    public CacheNode( String path, boolean isNew ) {

        this.path = path;

        this.isNew = isNew;

        this.directory = new File( this.getPath() ).getParent();
        
    }

    /**
     * Get the path to the data.xml file.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
     */
    public String getPath() { 
        
        return this.path;
        
    }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
     */
    public boolean isNew() {

        return this.isNew;

    }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
     */
    public void setIsNew( boolean isNew ) {

        this.isNew = isNew;
        
    }
    
    /**
     * Require that this cache node exists including the data.xml file and
     * associated directories.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: CacheNode.java,v 1.4 2001/11/24 10:14:44 burton Exp $
     */
    public void require() throws IOException {
        
        //make sure basedir exists
        new File( this.getDirectory() ).mkdirs();

        //make sure we have a data.xml file here.
        CacheConstructor.constructXML( this.getDirectory() );        
        
    }
    
    public String getDirectory() {

        return this.directory;

    }

}

