/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;

import java.io.*;
import java.util.*;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.event.*;
import org.openprivacy.panther.implementations.memory.*;
import org.openprivacy.panther.implementations.operations.*;
import org.openprivacy.panther.implementations.xml.*;
import org.openprivacy.panther.util.*;

import javax.xml.transform.*;

/**
 * Remote proxy interface for URL/resource caching/fetching.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
 */
public class PantherProxy extends BaseComponent implements Component, Proxy {

    protected static Proxy proxyInstance = null;
    
    private ResourceMapper mapper = null;

    private URIResolver uriResolver = null;
    
    public void init() throws TalonException {
    
        //add init code here

        this.uriResolver = new PantherURIResolver( this );

        proxyInstance = this;
        
    }

    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public Resource getResource( URI uri ) throws IOException {

        return mapper.getResource( uri );
        
    }

    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public Resource getResource( String uri ) throws IOException {

        return mapper.getResource( uri );
        
    }


    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void setResourceMapper( ResourceMapper mapper ) {

        this.mapper = mapper;

    }


    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ResourceMapper getResourceMapper() {

        return this.mapper;

    }

    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry get( String location ) throws IOException {
        

        return get( new URI( location ) );

    }


    
    
    /**
     * The default implementation of the Proxy.get() method.
     *
     * This method adds the behavior that if a given location is not in the
     * cache, it asks it to be updated.
     * 
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry get( URI location ) throws IOException {

        return Get.getInstance( this ).get( location );
        
    }

    /**
     * @see Proxy#update
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void update( String location ) throws IOException {

        UpdateManager.getInstance().update( location );
        
    }

    /**
     * @see Proxy#update
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void update( URI location ) throws IOException {

        update( location.toString() );
        
    }

    /**
     * @see Proxy#supports
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public boolean supports( URI location ) {

        return location.getScheme().equals( "http" ) || location.getScheme().equals( "ftp" );

    }

    /**
     * @see Proxy#supports
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public boolean supports( String location ) {

        return supports( new URI( location ) );

    }
    
    /**
     * @see Proxy#contains
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public boolean contains( URI location ) {

        return contains( location.toString() );
        
    }

    /**
     * @see Proxy#contains
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public boolean contains( String location ) {

        return Contains.getInstance( this ).contains( location );

    }

    /**
     * @see Proxy#getURIResolver
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public URIResolver getURIResolver() {

        return this.uriResolver;
        
    }
    
    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void asyncUpdate( String location ) {

        try { 

            //panther does async updates anyway.

            update( location );

        } catch ( IOException ioe ) {
            
            ioe.printStackTrace();
            
        }

        
    }

    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void asyncUpdate( URI location ) {

        asyncUpdate( location.toString() );

    }

    /**
     * @see ProxyEventListener
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void addProxyEventListener( ProxyEventListener listener ) {

        ProxyEventManager.getInstance().addProxyEventListener( listener );

    }

    /**
     * @see ProxyEventListener
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public void removeProxyEventListener( ProxyEventListener listener ) {

        ProxyEventManager.getInstance().removeProxyEventListener( listener );
        
    }

    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry getAndWait( URI location ) throws Exception {

        return Get.getInstance( this ).getAndWait( location );
        
    }

    
    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry getAndWait( String location ) throws Exception {

        return Get.getInstance( this ).getAndWait( new URI( location ) );

    }

    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry updateAndWait( URI location ) throws Exception {

        return Update.getInstance( this ).updateAndWait( location );
        
    }

    
    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxy.java,v 1.24 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry updateAndWait( String location ) throws Exception {

        return Update.getInstance( this ).updateAndWait( new URI( location ) );

    }


    
}

