/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;

import java.io.*;
import java.util.*;
import java.net.URL;

import talon.*;
import talon.components.*;
import talon.resources.*;
import talon.util.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.factories.*;
import org.openprivacy.panther.implementations.xml.*;

/**
 * Represents a URL/etc fetched from the proxy.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
 */
public class PantherProxyEntry implements ProxyEntry {
    
    /**
     * True if this is based on a resource URL.
     */
    private boolean resource = false;
    
    private long minCachedInterval = UpdateManager.DEFAULT_MIN_CACHED_INTERVAL;

    private URI identifier = null;

    //used if we need to load from a REAL URL instead of from the proxy cache.
    //This is generally used for resources.
    private URL fromURL = null;
    
    private String filename = null;
    
    private long lastModified = 0;

    /**
     * Last time (in miliseconds) which the Proxy tried to update this URL.
     */
    private long lastUpdateAttempt = 0;
    
    private String lastMessage;

    /*
     * Stores the cache file for this entry
     */
    private File file = null;

    private Proxy proxy = null;

    private Logger logger = Log.getLogger( this );

    private MimeType mimeType = null;
    
    /**
     * Common constructor...
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    private PantherProxyEntry( URI identifier ) {

        this.proxy = PantherProxy.proxyInstance;
        this.identifier = identifier;
        
    }
    
    /**
     * Create an Entry from a resource:/ URL.  
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public PantherProxyEntry( URI identifier,
                              Resource resource ) throws IOException {

        this( identifier );

        URL location = resource.getURL();
        
        //location as a string..
        String slocation = location.toString();
        
        if ( slocation.startsWith( "file:" ) ) {

            String filename = slocation.substring( "file:".length(), slocation.length() );
            
            this.lastModified = new File( filename ).lastModified();
            
        } else {

            this.lastModified = location.openConnection().getLastModified();
             
        }

        this.fromURL = location;
        
    }


    /**
     * Create an Entry from a resource:/ URL.  
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public PantherProxyEntry( URI identifier,
                              File file ) throws IOException {

        this( identifier );
        
        this.lastModified = file.lastModified();

        this.fromURL = new URL( identifier.toString() );
        
    }

    
    /**
     * Create a ProxyEntry from an item in the cache.  This should only be
     * called for local URLs AKA file://
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public PantherProxyEntry( URI identifier,
                              String filename,
                              long lastModified,
                              long minCachedInterval,
                              String mimeType ) {

        this( identifier );
        
        this.filename = filename;
        this.file = new File( filename );

        Assert.assert( lastModified != 0, "lastModified may not be 0" );
        
        this.lastModified = lastModified;

        this.minCachedInterval = minCachedInterval;

        this.mimeType = new MimeType( mimeType );
        
    }
    
    /**
     * Get an InputStream for reading from this ProxyEntry.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public InputStream getInputStream() throws IOException {

        //if we don't have a filename... return an InputStream from the URL...
        if ( this.fromURL != null ) {

            return this.fromURL.openStream();
            
        } 
        
        return new FileInputStream( filename );
        
    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public URI getIdentifier() {

        return this.identifier;

    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public long getLastModified() {

        return this.lastModified;
        
    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public String getLastMessage() {

        return this.lastMessage;

    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public void setLastMessage( String lastMessage ) {

        this.lastMessage = lastMessage;
        
    }
    
    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public String getFileLocation() {
        
        return "file://" + this.filename;
        
    }

    /**
     * @see Proxy
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public File getFile() throws IOException {

        return this.file;

    }
    
    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public boolean hasChanged() {

        //compare this to the latest entry the Proxy knows about.

        try { 
            
            ProxyEntry newEntry = proxy.get( identifier );
            
            return getLastModified() < newEntry.getLastModified();
            
        } catch ( Throwable t ) {

            logger.error( t );

            return false;

        }
        
    }

    /**
     * 
     * Get the value of <code>minCachedInterval</code>.
     *
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public long getMinCachedInterval() { 
        
        return this.minCachedInterval;
        
    }

    /**
     * 
     * Set the value of <code>minCachedInterval</code>.
     *
     * @see ProxyEntry
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public void setMinCachedInterval( long minCachedInterval ) { 
        
        this.minCachedInterval = minCachedInterval;
        
    }

    /**
     * 
     * Set the value of <code>resource</code>.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public void setResource( boolean resource ) { 
        
        this.resource = resource;
        
    }

    /**
     * 
     * Get the value of <code>resource</code>.
     * 
     * This is true if this ProxyEntry is based on a resource.
     * 
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public boolean isResource() { 
        
        return this.resource;
        
    }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: PantherProxyEntry.java,v 1.16 2002/02/27 07:48:29 burton Exp $
     */
    public void save() throws IOException {

        try { 

            new UpdateHandler( this.getIdentifier().toString(), false );
            
        } catch ( Exception e ) {

            e.printStackTrace();
            
            logger.error( e );
            throw new ProxyException( e.getMessage() );
            
        }

    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public MimeType getMimeType() {

        return this.mimeType;
        
    }

    /**
     * @see ProxyEntry
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public void setMimeType( MimeType mimeType ) {

        this.mimeType = mimeType;
        
    }

}

