/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.implementations.xml.*;
import org.openprivacy.panther.components.*;


/**
 * <p>
 * This class (singleton) tries to organize and cache ProxyEntry(s) so that
 * other classes within the package can pull out the previous known version.
 *
 * <p>EXTERNAL APPLICATION SHOULD NOT USE THIS CLASS
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: ProxyEntryManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
 */
public class ProxyEntryManager {

    private Hashtable entries = new Hashtable();

    private static ProxyEntryManager instance = null;

    
    /**
     * Get a ProxyEntry from the ProxyEntryManager.  Note that this can throw an
     * IOException if it can't find a ProxyEntry.  This is likely to happen if
     * it isn't in the cache.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEntryManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public ProxyEntry get( String location ) throws IOException {


        //if the given location is already know, just return it.  else we should
        //fetch it.

        if ( entries.containsKey( location ) ) {

            PantherProxyEntry entry = (PantherProxyEntry)entries.get( location );

            requireExists( entry );

            return entry;
            
        } else {

            FetchHandler fh = new FetchHandler( location );

            if ( fh.found() ) {

                ProxyEntry entry = fh.getProxyEntry();

                //make sure to add it to the ProxyEntryManager
                ProxyEntryManager.getInstance().set( location, entry );

                requireExists( entry );
                
                return entry;
                
            } else {
                    
                throw new ProxyException( "Update just ran, but URL was not in the cache." );
                
            }
            
            
        }
        
    }

    /**
     * Require that the entry is backed on disk.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEntryManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    private void requireExists( ProxyEntry entry ) throws IOException {

        if ( entry.getFile().exists() == false )
            throw new ProxyException( "Entry not present on disk: " + entry.getFile().getAbsolutePath() );
        
    }
    
    /**
     * Set a ProxyEntry in the ProxyEntryManager.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEntryManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public void set( String location, ProxyEntry entry ) {

        this.entries.put( location, entry );
        
    }

    /**
     * Get an instance of the ProxyEntryManager.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: ProxyEntryManager.java,v 1.7 2001/08/09 23:52:46 burton Exp $
     */
    public static ProxyEntryManager getInstance() {

        if ( instance == null ) { 

            instance = new ProxyEntryManager();

        }

        return instance;
    }


    
}    
