/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations.operations;

import java.io.*;
import java.util.*;
import java.net.URL;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.implementations.memory.*;
import org.openprivacy.panther.implementations.xml.*;
import org.openprivacy.panther.util.*;

/**
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: Get.java,v 1.4 2002/03/08 21:42:54 burton Exp $
 */
public class Get {
    
    /**
     * Instance member for <code>Get</code>
     */
    private static Get instance = null;

    private Proxy proxy = null;

    /**
     * 
     * Create a new <code>Get</code> instance.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Get.java,v 1.4 2002/03/08 21:42:54 burton Exp $
     */
    public Get( Proxy proxy ) {

        this.proxy = proxy;
        
    }
    
    
    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Get.java,v 1.4 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry get( URI location ) throws IOException {

        //if this is a resource URI then hse the resource mapper to get it.
        if ( location.getScheme().equals( "resource" ) ) {

            //need to rewrite the URL so that it has the "resource:" portion removed

            return new PantherProxyEntry( location,
                                          proxy.getResource( location ) );

        } 
        
        //if it is a file URL we can support it right away...
        if ( location.getScheme().equals( "file" ) ) {

            return new PantherProxyEntry( location,
                                          FileURL.toFile( location.toString() ) );
            
        }
        
        //if we support this url... 
        if ( proxy.supports( location ) )  {

            //try to get the cache location... 

            try {

                ProxyEntryManager pm = ProxyEntryManager.getInstance();

                ProxyEntry entry = pm.get( location.toString() );

                return entry;
                
            } catch ( IOException e ) {

                //thrown if the ProxyManager can't get the URL.. this should
                //request that it is updated.

                UpdateManager.getInstance().update( location.toString() );

                throw new ProxyException( "URL not proxied: " + location.toString() + " - " + e.getMessage() );

            } 
            
        }

        throw new ProxyException( "URL not supported: " + location.toString() );

    }


    /**
     * @see Proxy#get
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Get.java,v 1.4 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry getAndWait( URI location ) throws Exception {

        //if this is a resource URI then hse the resource mapper to get it.
        if ( location.getScheme().equals( "resource" ) ) {

            //need to rewrite the URL so that it has the "resource:" portion removed

            return new PantherProxyEntry( location,
                                          proxy.getResource( location ) );

        } 

        //if it is a file URL we can support it right away...
        if ( location.getScheme().equals( "file" ) ) {

            return new PantherProxyEntry( location,
                                          FileURL.toFile( location.toString() ) );
            
        }
        
        //if we support this url... 
        if ( proxy.supports( location ) )  {

            //try to get the cached location... if it is already in the
            //cache... we just return this version.

            try {

                ProxyEntryManager pm = ProxyEntryManager.getInstance();

                ProxyEntry entry = pm.get( location.toString() );

                //it is already in the cache... keep it local man.
                
                return entry;
                
            } catch ( IOException e ) {

                //thrown if the ProxyManager can't get the URL.. this should
                //request that it is updated.

                //it is NOT in the

                //FIXME: how do we wait here???

                synchronized( this ) {
                
                    UpdateHandler uh = new UpdateHandler( location.toString() );

                    return uh.getProxyEntry();
                    
                }
                
            } 
            
        }

        throw new ProxyException( "URL not supported: " + location.toString() );

    }


    /**
     * 
     * Get an instance of the <code>Get</code>.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Get.java,v 1.4 2002/03/08 21:42:54 burton Exp $
     */
    public static Get getInstance( Proxy proxy ) {
        
        if ( instance == null ) {
            
            instance = new Get( proxy );
            
        }
        
        return instance;
        
    }
    
}

