/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations.operations;

import java.io.*;
import java.util.*;
import java.net.URL;

import talon.*;
import talon.resources.*;
import talon.util.net.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.implementations.memory.*;
import org.openprivacy.panther.implementations.xml.*;
import org.openprivacy.panther.util.*;

/**
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: Update.java,v 1.1 2002/03/08 21:42:54 burton Exp $
 */
public class Update {
    
    /**
     * Instance member for <code>Update</code>
     */
    private static Update instance = null;

    private Proxy proxy = null;

    /**
     * 
     * Create a new <code>Update</code> instance.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Update.java,v 1.1 2002/03/08 21:42:54 burton Exp $
     */
    public Update( Proxy proxy ) {

        this.proxy = proxy;
        
    }

    /**
     * @see Proxy#updateAndWait
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Update.java,v 1.1 2002/03/08 21:42:54 burton Exp $
     */
    public ProxyEntry updateAndWait( URI location ) throws Exception {

        //if this is a resource URI then hse the resource mapper to get it.
        if ( location.getScheme().equals( "resource" ) ) {

            //need to rewrite the URL so that it has the "resource:" portion removed

            return new PantherProxyEntry( location,
                                          proxy.getResource( location ) );

        } 

        //if it is a file URL we can support it right away...
        if ( location.getScheme().equals( "file" ) ) {

            return new PantherProxyEntry( location,
                                          FileURL.toFile( location.toString() ) );
            
        }
        
        //if we support this url... 
        if ( proxy.supports( location ) )  {

            //how do we wait here???

            synchronized( this ) {
                
                UpdateHandler uh = new UpdateHandler( location.toString() );
                
                return uh.getProxyEntry();
                
            }
                
        }

        throw new ProxyException( "URL not supported: " + location.toString() );

    }


    /**
     * 
     * Update an instance of the <code>Update</code>.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Update.java,v 1.1 2002/03/08 21:42:54 burton Exp $
     */
    public static Update getInstance( Proxy proxy ) {
        
        if ( instance == null ) {
            
            instance = new Update( proxy );
            
        }
        
        return instance;
        
    }
    
}

