/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */


package org.openprivacy.panther.implementations.xml;

import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.util.*;
import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.implementations.memory.*;

import talon.*;
import talon.util.*;
import talon.util.net.*;

/**
 * Responsible for fetching content form the cache.
 * 
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: FetchHandler.java,v 1.6 2001/11/24 10:14:44 burton Exp $
 */
public class FetchHandler extends DefaultHandler {

    private String uri = "";

    private String local = "";

    private String findLocation = "";

    private String content = "";

    private boolean found = false;

    /**
     * Filename of the cache data...
     */
    private String filename = "";

    private long lastModified = 0;

    private long minCachedInterval = 0;

    private String mimeType = "";
    
    /**
     * Init a FetchHandler responsible for finding the given location.   
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: FetchHandler.java,v 1.6 2001/11/24 10:14:44 burton Exp $
     */
    public FetchHandler( String findLocation ) throws IOException {

        CacheNode node = CacheConstructor.construct( findLocation );

        this.findLocation = findLocation;

        //make sure the node file exists...  if not... don't do anything..
        if ( new File( node.getPath() ).exists() ) {

            try {
        
                XMLReader xmlr = XMLReaderFactory.createXMLReader( PantherResources.PARSER );

                xmlr.setContentHandler( this );
        
                xmlr.parse( node.getPath() );

            } catch ( SAXException e ) {

                e.printStackTrace();
                throw new ProxyException( e.getMessage() );

            }

        }         
        
    }
    
    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: FetchHandler.java,v 1.6 2001/11/24 10:14:44 burton Exp $
     */
    public void startElement( String uri,
                              String local,
                              String raw,
                              Attributes attrs) {
        
        this.uri = uri;

        this.local = local;

        if ( attrs != null &&
             uri.equals( PantherResources.CACHE_NAMESPACE ) &&
             local.equals( "entry" ) ) {

            String location = attrs.getValue( "location" );

            String filename = attrs.getValue( "filename" );

            String mimeType = attrs.getValue( "mimeType" );
            
            if ( location.equals( findLocation ) &&
                 filename != null ) {

                this.found = true;

                this.filename = filename;

                this.lastModified = Long.parseLong( attrs.getValue( "lastModified" ) );

                this.mimeType = mimeType;
                
                String attr_minCachedInterval = attrs.getValue( "minCachedInterval" );

                if ( attr_minCachedInterval != null ) {
                    this.minCachedInterval = Long.parseLong( attr_minCachedInterval );
                } else {
                    this.minCachedInterval = UpdateManager.DEFAULT_MIN_CACHED_INTERVAL;
                }
                
                Assert.assert( lastModified != 0, "lastModified may not be 0: " + lastModified );
                
            } 
            
        } 
        
    } 

    
    public void endElement( String uri,
                            String localName,
                            String qName ) {

        this.local = "";            
        
    }
    
    /**
     * Return true if this URL was found.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: FetchHandler.java,v 1.6 2001/11/24 10:14:44 burton Exp $
     */
    public boolean found() {

        return this.found;

    }

    /**
     * Build a ProxyEntry from this which reflect the latest update.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: FetchHandler.java,v 1.6 2001/11/24 10:14:44 burton Exp $
     */
    public ProxyEntry getProxyEntry() {

        return new PantherProxyEntry( new URI( findLocation ),
                                      filename,
                                      lastModified,
                                      minCachedInterval,
                                      mimeType );
                               
    }
    
}

