/*
 *
 * BEGIN HEADER
 *
 * ---- 
 *
 * $ID: PantherProxy.java,v 1.6 2001/06/12 20:58:43 burton Exp $
 * $Project: http://panther.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/panther/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * ---- 
 *
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 *
 * END HEADER
 * 
 */

package org.openprivacy.panther.implementations.xml;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import java.security.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

import org.openprivacy.panther.components.*;
import org.openprivacy.panther.event.*;
import org.openprivacy.panther.implementations.*;
import org.openprivacy.panther.implementations.memory.*;
import org.openprivacy.panther.util.*;

import talon.*;
import talon.util.*;
import talon.util.net.*;

/**
 * <p> Responsible for finding a cache entry and updating it (note that the
 * UpdateManager actually handles if a URL *should* be updated.
 * 
 * <p> If the URL is found inline then we will update it there.  Else we will
 * insert it at the bottom when <entry> ends...
 * 
 * <p> NOTE: I didn't spend too much time trying to make this output correct XML
 * in all cases.  If the data.xml file is modified significantly then this may
 * break.  This was done as to abvoid obfuscating this class due to XML
 * complexity.
 *
 * TODO:
 * 
 * - Is it possible to do this with JDOM so that we don't have to embed Java
 * code?
 * 
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
 */
public class UpdateHandler extends DefaultHandler {

    private ByteArrayOutputStream bos = new ByteArrayOutputStream();

    private PrintWriter out = new PrintWriter( bos );

    private String updateLocation = "";

    private String mimeType = "text/xml";
    
    //true if we updated content.
    private boolean updated = false;

    private byte[] content;;

    //datfile used to store the URL after we update it.  
    private String datfile = "";

    /**
     * The last time this URL was modified.
     */
    private long lastModified = 0;

    private long minCachedInterval = UpdateManager.DEFAULT_MIN_CACHED_INTERVAL;
        
    /**
     * True if we are going download new content.
     */
    private boolean download = false;

    //current parsing element information...
    private String currentNamespace = "";
    private String currentElement = "";
    private String currentLocation = "";    

    //the updated Proxyentry..
    private ProxyEntry entry = null;
    
    /**
     * Update this entry on disk but DO NOT download the latest copy.  This is
     * used so that we can make modification to an entries metainfo without
     * having to download new content.
     * 
     * @see #UpdateHandler
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    public UpdateHandler( String updateLocation,
                          boolean download ) throws Exception {

        this.download = download;
        
        //FIXME: provide an implementation!  it should hare the implementation
        //code with the other constructor.
        
        CacheNode node = CacheConstructor.construct( updateLocation );

        this.updateLocation = updateLocation;

        //This should not be a synchronized operation due to the fact that we
        //don't want to lock the UpdateManager on URLs which take a LONG time to
        //download.
        if ( download )
            this.content = getContent();

        XMLReader xmlr = XMLReaderFactory.createXMLReader( PantherResources.PARSER );

        xmlr.setContentHandler( this );

        node.require();
        
        //This this is the datafile for the current file for this node.

        //FIXME: use the current node if we are not going to download a new node.
        if ( download ) {

            datfile = node.getDirectory() + "/" + System.currentTimeMillis() + ".dat"; 

        } else {

            //resort to the current datafile.
            
            ProxyEntry entry = ProxyEntryManager.getInstance().get( updateLocation );

            Assert.assert( entry != null, "Could not find an entry for this location: " + updateLocation );

            this.datfile = entry.getFile().getAbsolutePath();            

            this.minCachedInterval = entry.getMinCachedInterval();
            
        }

        //everything from this point needs to be synchronized.  

        synchronized( UpdateManager.getInstance() ) {
        
            //save the datfile if necessary
            if ( download ) {

                FileOutputStream datos = new FileOutputStream( datfile );
                datos.write( content );
                datos.flush();
                datos.close();
                
            } 

            String nodePath = node.getPath();

            Assert.assert( nodePath, "A node path was not specified" );
            
            //use the XML parer to update this node.
            xmlr.parse( nodePath );

            //flush the modifications.
            out.flush();
            bos.flush();

            //save the content to the data.xml node file...

            node.setIsNew( false );

            //Write the modified XML back into the node file.
            
            FileOutputStream fos = new FileOutputStream( node.getPath() );
        
            fos.write( bos.toByteArray() );

            fos.flush();
            fos.close();

        }

        //make a new ProxyEntry

        this.entry = new PantherProxyEntry( new URI( updateLocation ),
                                            datfile,
                                            lastModified,
                                            minCachedInterval,
                                            mimeType );

        //update the known entry within the manager
        ProxyEntryManager.getInstance().set( updateLocation, entry );
        
        //fire off an event that this was updated/downloaded
        if ( download )
            ProxyEventManager.getInstance().fireUpdated( entry );
        
    }
    
    /**
     * Init a FetchHandler responsible for finding the given location and
     * updating it...
     * 
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    public UpdateHandler( String updateLocation ) throws Exception {

        this( updateLocation, true );
        
    }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    public void startElement( String uri,
                              String local,
                              String raw,
                              Attributes attrs) {

        this.currentNamespace = uri;
        this.currentElement = local;
        
        //FIXME: since this is an update, if an existing <entry> is here with
        //the location we are going to update we need to remove this as it would
        //accidentally make a duplicate.  Get the filename and delete it and
        //then return,

        if ( uri.equals( PantherResources.CACHE_NAMESPACE ) &&
             local.equals( "entry" ) &&
             attrs.getValue( "location" ) != null ) {

            String location = attrs.getValue( "location" );

            this.currentLocation = location;
            
            if ( location.equals( updateLocation ) ) {
                
                //opps... we found a duplicate.. delete the file and return.
                new File( attrs.getValue( "filename" ) ).delete();

                if ( download == false ) {

                    String attr_lastModified = attrs.getValue( "lastModified" );
                    
                    if ( attr_lastModified != null )
                        this.lastModified = Long.valueOf( attr_lastModified ).longValue();
                    
                }                

                //return so that this is element not copied

                return;

            } 

        } 
        
        out.print('<');
        out.print( raw );

        if (attrs != null) {

            //attrs = sortAttributes(attrs);
            int len = attrs.getLength();
            for ( int i = 0; i < len; i++ ) {

                out.print( ' ' );
                out.print( attrs.getQName( i ) );
                out.print( "=\"" );
                out.print( normalize( attrs.getValue( i ) ) );
                out.print( '"' );

            }

        }

        out.print( " xmlns=\"" + uri + "\"" );
        
        out.println('>');
        
    } 

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    public void endElement( String uri,
                            String local,
                            String qName ) {

        if ( updated == false &&
             uri.equals( PantherResources.CACHE_NAMESPACE ) &&
             local.equals( "cache" ) ) {

            //update the element....  This requires a new entry and a data
            //element

            //FIXME: What other things do we have to escape???
            String escapedLocation = StringUtils.replaceAll( updateLocation, "&", "&amp;" );

            //use the lastModified date if we haven't determiend a new one.
            if ( lastModified == 0 )
                lastModified = System.currentTimeMillis();                
            
            out.println( "<entry location=\"" + escapedLocation + "\" " +
                                "lastModified=\"" + lastModified + "\" " +
                                "mimeType=\"" + mimeType + "\" " +
                                "minCachedInterval=\"" + minCachedInterval + "\" " +
                                "filename=\"" + datfile + "\"/>" );
            
        }

        if ( ! ( currentNamespace.equals( PantherResources.CACHE_NAMESPACE ) &&
                 currentElement.equals( "entry" ) &&
                 currentLocation.equals( updateLocation ) &&
                 local.equals( "entry" ) ) ) {

            //output and end element for this. 
            out.println( "</" + local + ">" );
            
        } 
        
    }

    /**
     * Normalize the given string so that it can be stored within XML.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    private String normalize(String s) {
        
        StringBuffer str = new StringBuffer();

        int len = ( s != null ) ? s.length() : 0;

        for ( int i = 0; i < len; i++ ) {
        
            char ch = s.charAt(i);
        
            switch (ch) {
                case '<': {
                    str.append("&lt;");
                    break;
                }
                case '>': {
                    str.append("&gt;");
                    break;
                }
                case '&': {
                    str.append("&amp;");
                    break;
                }
                case '"': {
                    str.append("&quot;");
                    break;
                }
                default: {
                    str.append(ch);
                }

            }

        }

        return str.toString();

    } 

    /**
     * Get the content from the remote site and return it.  
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: UpdateHandler.java,v 1.11 2002/07/17 03:56:01 burton Exp $
     */
    private byte[] getContent() throws IOException {

        String location = this.updateLocation;
        
        //figure out if we should escape the URL...
        if ( location.indexOf( "?" ) != -1 ) {
            
            //need to encode DATA past the "?" before we proceed.
            String url = updateLocation.substring( 0, updateLocation.indexOf( "?" ) );

            String urlData = updateLocation.substring( updateLocation.indexOf( "?" ) + 1, updateLocation.length() );

            urlData = StringUtils.replaceAll( urlData, " ", "%20" );
        
            //urlData = URLEncoder.encode( urlData );
        
            location = url + "?" + urlData;

        } 

        URL url = new URL( location );

        URLConnection urlc = url.openConnection();

        //OK.  if this is an HTTP URL, We can ONLY accept this this if we have a
        //202 status code.  This turned out to be a significant problem in
        //real-world situations.
        if ( url.getProtocol().equals( "http" ) ) {

            HttpURLConnection hurlc = (HttpURLConnection)urlc;

            //update the mime type...

            this.mimeType = hurlc.getContentType();
            
            if ( hurlc.getResponseCode() != HttpURLConnection.HTTP_OK ) {

                String message = hurlc.getResponseMessage();
                
                Log.error( message );
                
                throw new IOException( message );
                
            } 

        } 

        InputStream is = urlc.getInputStream();

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
      
        //now process the Reader...
        byte data[] = new byte[200];
    
        int readCount = 0;

        while( ( readCount = is.read( data )) > 0 ) {
            
            bos.write( data, 0, readCount );
        }

        is.close();
        bos.close();

        //update the memory store.
        byte[] content = bos.toByteArray();

        //make sure we keep this in memory...
        MemoryCache.store( this.updateLocation, content );

        return content;
        
    }

    public ProxyEntry getProxyEntry() {

        return this.entry;
        
    }
    
}

